@@ -2,7 +2,7 @@ package VIC::Grammar;
 use strict;
 use warnings;
 
-our $VERSION = '0.23';
+our $VERSION = '0.24';
 $VERSION = eval $VERSION;
 
 use Pegex::Base;
@@ -10,7 +10,7 @@ extends 'Pegex::Grammar';
 
 use constant file => './share/vic.pgx';
 
-sub make_tree {   # Generated/Inlined by Pegex::Grammar (0.57)
+sub make_tree {   # Generated/Inlined by Pegex::Grammar (0.60)
   {
     '+grammar' => 'vic',
     '+toprule' => 'program',
@@ -413,6 +413,9 @@ sub make_tree {   # Generated/Inlined by Pegex::Grammar (0.57)
             },
             {
               '.ref' => 'modifier_constant'
+            },
+            {
+              '.ref' => 'parameter'
             }
           ]
         }
@@ -654,6 +657,9 @@ sub make_tree {   # Generated/Inlined by Pegex::Grammar (0.57)
         }
       ]
     },
+    'parameter' => {
+      '.rgx' => qr/\Gshift/
+    },
     'pragma_expression' => {
       '.all' => [
         {
