@@ -1,7 +1,7 @@
 package VIC::PIC::Functions::CodeGen;
 use strict;
 use warnings;
-our $VERSION = '0.23';
+our $VERSION = '0.24';
 $VERSION = eval $VERSION;
 use Carp;
 use POSIX ();
@@ -24,6 +24,14 @@ has code_config => (is => 'rw', default => sub {
                 bits => 8, # bits. same as register_size
                 export => 0, # do not export variables
             },
+            uart => {
+                baud => 9600, # baud rate
+                bit9 => 0, # allow 9 bits
+            },
+            usart => {
+                baud => 9600, # baud rate
+                bit9 => 0, # allow 9 bits
+            },
         }
 });
 
@@ -34,6 +42,7 @@ sub validate {
     return 0 unless $self->doesrole('Chip');
     return 1 if exists $self->pins->{$var};
     return 1 if exists $self->registers->{$var};
+    return 1 if ($self->doesrole('USART', 1) and exists $self->usart_pins->{$var});
     return 0;
 }
 
@@ -61,6 +70,7 @@ sub update_code_config {
     my ($self, $grp, $key, $val) = @_;
     return unless $self->doesrole('CodeGen');
     return unless defined $grp;
+    $grp = lc $grp; # force lower case in case of usage for things like SPI/UART/I2C
     $self->code_config->{$grp} = {} unless exists $self->code_config->{$grp};
     my $grpref = $self->code_config->{$grp};
     if ($key eq 'bits') {
@@ -90,5 +100,14 @@ sub address_bits {
     return $bits;
 }
 
+sub is_variable {
+    my ($self, $varname) = @_;
+    return unless $varname;
+    return unless $self->doesrole('CodeGen');
+    return 1 if defined $self->code_config->{lc $varname};
+    return 0;
+}
+
+
 1;
 __END__
