@@ -2,10 +2,11 @@ package VIC::PIC::Functions::GPIO;
 use strict;
 use warnings;
 use bigint;
-our $VERSION = '0.23';
+our $VERSION = '0.24';
 $VERSION = eval $VERSION;
 use Carp;
 use POSIX ();
+use Scalar::Util qw(looks_like_number);
 use Moo::Role;
 
 sub get_output_pin {
@@ -49,7 +50,8 @@ sub get_input_pin {
 }
 
 sub _gpio_select {
-    my ($self, $io, $ad, $outp) = @_;
+    my $self = shift;
+    my ($io, $ad, $outp) = @_;
     return unless $self->doesroles(qw(Chip GPIO));
     return unless defined $outp;
     $io = 0 if $io =~ /output/i;
@@ -180,8 +182,21 @@ sub analog_input {
     return $self->_gpio_select(input => 'analog', @_);
 }
 
+sub setup {
+    my $self = shift;
+    my ($outp) = @_;
+    if ($outp =~ /US?ART/) {
+        if ($self->doesrole('USART') and exists $self->usart_pins->{$outp}) {
+            return $self->usart_setup(@_);
+        }
+    }
+    carp "The 'setup' function is not valid for $outp. Use something else.";
+    return;
+}
+
 sub write {
-    my ($self, $outp, $val) = @_;
+    my $self = shift;
+    my ($outp, $val) = @_;
     return unless $self->doesroles(qw(CodeGen Operations Chip GPIO));
     return unless defined $outp;
     if (exists $self->io_ports->{$outp} and
@@ -202,6 +217,7 @@ sub write {
                 return;
             }
         }
+        # this handles the variable to port assigning
         return $self->op_assign($outp, $val);
     } elsif (exists $self->pins->{$outp}) {
         my $iopin = $self->get_output_pin($outp);
@@ -232,6 +248,15 @@ sub write {
                     "Only a pin can be written to a pin.\n";
                 return;
             }
+        } elsif ($self->is_variable($val)) {
+            $val = uc $val;
+            return << "...";
+;;;; assigning $val to a pin => using the last bit
+\tbtfss $val, 0
+\tbcf $port, $outp
+\tbtfsc $val, 0
+\tbsf $port, $outp
+...
         } else {
             carp "$val is a port or unknown pin and cannot be written to a pin $outp. ".
             "Only a pin can be written to a pin.\n";
@@ -243,10 +268,136 @@ sub write {
         $code .= $self->op_assign($outp, $val);
         return $code;
     } else {
+        if ($self->doesrole('USART') and exists $self->usart_pins->{$outp}) {
+            return $self->usart_write(@_);
+        }
         carp "Cannot find $outp in the list of ports, register or pins to write to";
         return;
     }
 }
 
+sub _macro_read_var {
+    my $v = $_[1];
+    $v = uc $v;
+    return << "...";
+;;;;;;; $v VARIABLES ;;;;;;
+$v\_UDATA udata
+$v res 1
+...
+}
+
+sub read {
+    my $self = shift;
+    my $inp = shift;
+    my $var = undef;
+    my %action = ();
+    if (scalar(@_) == 1) {
+        $var = shift;
+    } else {
+        %action = @_;
+    }
+    return unless $self->doesroles(qw(CodeGen Chip GPIO));
+    my ($code, $funcs, $macros, $tables) = ('', {}, {}, []);
+
+    if (defined $var) {
+        if (looks_like_number($var) or ref $var eq 'HASH') {
+            carp "Cannot read from $inp into a constant $var";
+            return;
+        }
+        $var = uc $var;
+    } else {
+        ## we need only 1 variable here
+        if (defined $action{PARAM}) {
+            $var = $action{PARAM} . '0';
+        } else {
+            carp "Implementation errors implementing the Action block";
+            return undef;
+        }
+        $var = uc $var;
+        $macros->{lc("m_read_$var")} = $self->_macro_read_var($var);
+        return unless (defined $action{ACTION} or defined $action{ISR});
+        return unless defined $action{END};
+    }
+    my $bits = $self->address_bits($var);
+    my ($port, $portbit);
+    if (exists $self->io_ports->{$inp} and
+        exists $self->registers->{$inp}) {
+        # this is a port like PORT[A-Z]
+        # we may end up reading from all pins on a port
+        $port = $inp;
+        $code = <<"...";
+;;; instant reading from $port into $var
+\tbanksel $port
+\tmovf $port, W
+\tbanksel $var
+\tmovwf $var
+...
+    } elsif (exists $self->pins->{$inp}) {
+        my $ipin = $self->get_input_pin($inp);
+        unless (defined $ipin) {
+            carp "Cannot find $inp in the list of GPIO ports or pins";
+            return;
+        } else {
+            my $tris;
+            ($port, $tris, $portbit) = @{$self->input_pins->{$inp}};
+            $code = <<"....";
+;;; instant reading from $inp into $var
+\tclrw
+\tbanksel $port
+\tbtfsc $port, $portbit
+\taddlw 0x01
+\tbanksel $var
+\tmovwf $var
+....
+        }
+    } else {
+        if ($self->doesrole('USART') and exists $self->usart_pins->{$inp}) {
+            return $self->usart_read(@_);
+        }
+        carp "Cannot find $inp in the list of ports or pins to read from";
+        return;
+    }
+    if (%action) {
+        if (exists $action{ACTION}) {
+            my $action_label = $action{ACTION};
+            my $end_label = $action{END};
+            $code .= <<"...";
+;;; invoking $action_label
+\tgoto $action_label
+$end_label:\n
+...
+        } elsif (exists $action{ISR}) {
+            ## ok we can read from a port too, so let's do that as well
+            if (defined $portbit) {
+                # if we are a pin, then find the right pin
+                $inp = $self->get_input_pin($inp);
+            }
+            ## reset the code here since we have to check IOC pins
+            my ($ioc_bit, $ioc_reg, $ioc_flag, $ioc_enable);
+            if (exists $self->ioc_pins->{$inp}) {
+                my $apin;
+                ($apin, $ioc_bit, $ioc_reg) = @{$self->ioc_pins->{$inp}};
+            } elsif (exists $self->ioc_ports->{$inp}) {
+                $ioc_reg = $self->ioc_ports->{$inp};
+            } else {
+                carp "Reading using interrupt-on-change has to be for a pin ".
+                        "that supports it, $inp does not support it or is not a pin.";
+                return;
+            }
+            $ioc_flag = $self->ioc_ports->{FLAG};
+            $ioc_enable = $self->ioc_ports->{ENABLE};
+            my $ioch = { bit => $ioc_bit, reg => $ioc_reg, flag =>
+                    $ioc_flag, enable => $ioc_enable };
+            $code = $self->isr_ioc($ioch, $inp);
+            my $isr_label = 'isr_' . ((defined $ioc_bit) ? lc($ioc_bit) : lc($ioc_reg));
+            $funcs->{$isr_label} = $self->isr_ioc($ioch, $inp, $var, $port, $portbit, %action);
+        } else {
+            carp "Unknown action requested. Probably a bug in implementation";
+            return;
+        }
+    }
+    return wantarray ? ($code, $funcs, $macros, $tables) : $code;
+}
+
 1;
 __END__
