@@ -2,7 +2,7 @@ package VIC::PIC::Functions::Operators;
 use strict;
 use warnings;
 use bigint;
-our $VERSION = '0.23';
+our $VERSION = '0.24';
 $VERSION = eval $VERSION;
 use Carp;
 use POSIX ();
@@ -1579,6 +1579,29 @@ sub op_stridx {
     #XXX { string => $string, index => $idx, %extra };
 }
 
+sub store_bytes {
+    my ($self, $tables) = @_;
+    return unless defined $tables;
+    return unless $self->doesrole('Chip');
+
+    unless (ref $tables eq 'ARRAY') {
+        $tables = [ $tables ];
+    }
+    return '' unless scalar @$tables;
+    my $code = '';
+    foreach (@$tables) {
+        my $name = $_->{name};
+        next unless defined $name;
+        next unless exists $_->{bytes};
+        next unless ref $_->{bytes} eq 'ARRAY';
+        my $bytes = join(',', @{$_->{bytes}});
+        $code .= $_->{comment} . "\n" if defined $_->{comment};
+        my $row = "$name:\n\taddwf PCL, F\n\tdt $bytes\n";
+        $code .= $row;
+    }
+    return $code;
+}
+
 1;
 __END__
 
