@@ -5,7 +5,7 @@ use bigint;
 use Carp;
 use Pegex::Base; # use this instead of Mo
 
-our $VERSION = '0.23';
+our $VERSION = '0.24';
 $VERSION = eval $VERSION;
 
 has type => 'gpsim';
@@ -14,7 +14,7 @@ has include => 'coff.inc';
 
 has pic => undef; # refer to the PIC object
 
-has led_count => 0;
+has node_count => 0;
 
 has scope_channels => 0;
 
@@ -54,9 +54,13 @@ $freq
 sub _gen_led {
     my $self = shift;
     my ($id, $x, $y, $name, $port, $color) = @_;
+    if (defined $color and ref $color eq 'HASH') {
+        $color = $color->{string};
+    }
     $color = 'red' unless defined $color;
     $color = 'red' unless $color =~ /red|orange|green|yellow|blue/i;
     $color = lc $color;
+    $color = substr ($color, 1) if $color =~ /^@/;
     return << "...";
 \t.sim "module load led L$id"
 \t.sim "L$id.xpos = $x"
@@ -161,9 +165,9 @@ sub attach_led {
     $count = 1 if int($count) < 1;
     my $code = '';
     if ($count == 1) {
-        my $c = $self->led_count;
+        my $c = $self->node_count;
         my $node = lc $port . 'led';
-        $self->led_count($c + 1);
+        $self->node_count($c + 1);
         my $x = ($c >= 4) ? 400 : 100;
         my $y = 50 + 50 * $c;
         # use the default pin 0 here
@@ -173,14 +177,14 @@ sub attach_led {
         $count--;
         if ($self->pic) {
             for (0 .. $count) {
-                my $c = $self->led_count + $_;
+                my $c = $self->node_count + $_;
                 my $x = ($_ >= 4) ? 400 : 100;
                 my $y = 50 + 50 * $c;
                 my $node = lc $port . $c . 'led';
                 my $simport = $self->_get_simport($port, $_);
                 $code .= $self->_gen_led($c, $x, $y, $node, $simport, $color);
             }
-            $self->led_count($self->led_count + $count + 1);
+            $self->node_count($self->node_count + $count + 1);
         }
     }
     return $code;
@@ -192,6 +196,10 @@ sub attach_led7seg {
     my @simpins = ();
     my $color = 'red';
     foreach my $p (@pins) {
+        if (defined $p and ref $p eq 'HASH') {
+            $p = $p->{string};
+            next unless defined $p;
+        }
         if (exists $self->pic->pins->{$p}) {
             push @simpins, $p;
         } elsif (exists $self->pic->registers->{$p}) {
@@ -202,14 +210,15 @@ sub attach_led7seg {
             }
         } elsif ($p =~ /red|orange|green|yellow|blue/i) {
             $color = $p;
+            $color = substr($p, 1) if $p =~ /^@/;
             next;
         } else {
             carp "Ignoring port $p as it doesn't exist\n";
         }
     }
     return unless scalar @simpins;
-    my $id = $self->led_count;
-    $self->led_count($id + 1);
+    my $id = $self->node_count;
+    $self->node_count($id + 1);
     my $x = 500;
     my $y = 50 + 50 * $id;
     $code .= << "...";
@@ -242,6 +251,10 @@ sub stop_after {
 sub logfile {
     my ($self, $file) = @_;
     $file = "vicsim.log" unless defined $file;
+    if (ref $file eq 'HASH') {
+        $file = $file->{string} || 'vicsim.log';
+    }
+    $file = substr($file, 1) if $file =~ /^@/;
     return "\t.sim \"log lxt $file\"\n" if $file =~ /\.lxt/i;
     return "\t.sim \"log on $file\"\n";
 }
@@ -250,12 +263,24 @@ sub log {
     my $self = shift;
     my $code = '';
     foreach my $port (@_) {
-        my $reg = $self->_get_simreg($port);
-        next unless $reg;
-        $code .= << "...";
+        if ($port =~ /US?ART/) {
+            next unless $self->pic->doesrole('USART');
+            my $ipin = $self->pic->usart_pins->{async_in};
+            my $opin = $self->pic->usart_pins->{async_out};
+            if (defined $ipin and defined $opin) {
+                my $ireg = $self->_get_simreg($ipin);
+                my $oreg = $self->_get_simreg($opin);
+                $code .= $self->log($ipin);
+                $code .= $self->log($opin) if $ireg ne $oreg;
+            }
+        } else {
+            my $reg = $self->_get_simreg($port);
+            next unless $reg;
+            $code .= << "...";
 \t.sim "log r $reg"
 \t.sim "log w $reg"
 ...
+        }
     }
     return $code;
 }
@@ -291,7 +316,15 @@ sub scope {
     my $self = shift;
     my $code = '';
     foreach my $port (@_) {
-        $code .= $self->_set_scope($port);
+        if ($port =~ /US?ART/) {
+            next unless $self->pic->doesrole('USART');
+            my $ipin = $self->pic->usart_pins->{async_in};
+            my $opin = $self->pic->usart_pins->{async_out};
+            $code .= $self->_set_scope($ipin) if defined $opin;
+            $code .= $self->_set_scope($opin) if defined $opin;
+        } else {
+            $code .= $self->_set_scope($port);
+        }
     }
     return $code;
 }
@@ -351,14 +384,29 @@ sub sim_assert {
             $condition = "$lhs $op2 $rhs";
         }
         #TODO: handle more complex expressions
+        if (defined $msg and ref $msg eq 'HASH') {
+            $msg = $msg->{string};
+        }
         $msg  = "$condition is false" unless $msg;
+        $msg = substr($msg, 1) if $msg =~ /^@/;
+        $condition = substr($condition, 1) if $condition =~ /^@/;
         $assert_msg = qq{$condition, \\\"$msg\\\"};
     } else {
+        if (defined $msg and ref $msg eq 'HASH') {
+            $msg = $msg->{string};
+        }
+        if (defined $condition and ref $condition eq 'HASH') {
+            $condition = $condition->{string};
+        }
         if (defined $condition and defined $msg) {
+            $msg = substr($msg, 1) if $msg =~ /^@/;
+            $condition = substr($condition, 1) if $condition =~ /^@/;
             $assert_msg = qq{$condition, \\\"$msg\\\"};
         } elsif (defined $condition and not defined $msg) {
+            $condition = substr($condition, 1) if $condition =~ /^@/;
             $assert_msg = qq{\\\"$condition\\\"};
         } elsif (defined $msg and not defined $condition) {
+            $msg = substr($msg, 1) if $msg =~ /^@/;
             $assert_msg = qq{\\\"$msg\\\"};
         } else {
             $assert_msg = qq{\\\"user requested an assert\\\"};
@@ -380,8 +428,8 @@ sub stimulate {
         %hh = (%hh, %$href);
     }
     my $period = '';
-    $period = $hh{EVERY} if defined $hh{EVERY};
-    $period = qq{\t.sim "period $period"} if defined $period;
+    $period = $hh{EVERY} if (defined $hh{EVERY} and length $hh{EVERY});
+    $period = qq{\t.sim "period $period"} if (defined $period and length $period);
     my $wave = '';
     my $wave_type = 'digital';
     if (exists $hh{WAVE} and ref $hh{WAVE} eq 'ARRAY') {
@@ -435,6 +483,44 @@ sub stopwatch {
     return $code;
 }
 
+sub attach {
+    my $self = shift;
+    return unless @_;
+    my $pin = shift;
+    my $code = '';
+    if ($pin =~ /US?ART/) {
+        # TX - connect to UART
+        # RX - connect to UART but also send it data
+        unless ($self->pic->doesrole('USART')) {
+            carp "PIC ", $self->pic->type, " does not do USART";
+            return;
+        }
+        my $baudrate = shift if @_;
+        my $key = ($pin =~ /^UART/) ? 'uart' : 'usart';
+        $baudrate = $self->pic->code_config->{$key}->{baud} unless defined
+        $baudrate;
+        $baudrate = 9600 unless defined $baudrate;
+        my $id = $self->node_count;
+        $self->node_count($id + 1);
+        my $ipin = $self->pic->usart_pins->{async_in};
+        my $rxport = $self->_get_simport($ipin);
+        my $opin = $self->pic->usart_pins->{async_out};
+        my $txport = $self->_get_simport($opin);
+        return unless (exists $self->pic->pins->{$ipin} and exists $self->pic->pins->{$opin});
+        $code .= qq{\t.sim "module load usart U$id"\n};
+        $code .= qq{\t.sim "node TX_U$id"\n};
+        $code .= qq{\t.sim "node RX_U$id"\n};
+        $code .= qq{\t.sim "attach TX_U$id $txport U$id.RXPIN"\n};
+        $code .= qq{\t.sim "attach RX_U$id $rxport U$id.TXPIN"\n};
+        $code .= qq{\t.sim "U$id.txbaud = $baudrate"\n};
+        $code .= qq{\t.sim "U$id.rxbaud = $baudrate"\n};
+        my $x = 500;
+        my $y = 50 + 50 * $id;
+        $code .= qq{\t.sim "U$id.xpos = $x"\n};
+        $code .= qq{\t.sim "U$id.ypos = $y"\n};
+    }
+}
+
 1;
 
 =encoding utf8
