@@ -1,8 +1,9 @@
 package VIC::PIC::P16F627A;
 use strict;
 use warnings;
-our $VERSION = '0.23';
+our $VERSION = '0.24';
 $VERSION = eval $VERSION;
+use Carp;
 use Moo;
 extends 'VIC::PIC::Base';
 
@@ -275,9 +276,19 @@ has usart_pins => (is => 'ro', default => sub {
         async_out => 'TX',
         sync_clock => 'CK',
         sync_data => 'DT',
+        # this defines the port names that the user can use
+        # validly. The port names define whether the user wants to use them in
+        # synchronous or asynchronous mode
+        UART => 'async',
+        USART => 'sync',
     }
 });
 
+sub usart_baudrates {
+    carp "Unimplemented";
+    return;
+}
+
 has cmp_output_pins => (is => 'ro', default => sub {
     {
         CMP1 => 'CMP1',
