@@ -1,7 +1,7 @@
 package VIC::PIC::P16F631;
 use strict;
 use warnings;
-our $VERSION = '0.23';
+our $VERSION = '0.24';
 $VERSION = eval $VERSION;
 use Moo;
 extends 'VIC::PIC::Base';
@@ -240,8 +240,9 @@ has wdt_prescaler => (is => 'ro', default => sub {
 
 has timer_pins => (is => 'ro', default => sub {
     {
-        TMR0 => 'TMR0', # denotes 8-bit
-        TMR1 => ['TMR1H', 'TMR1L'], # denotes 16-bit
+        #reg     #reg      #ireg #flag  #enable
+        TMR0 => { reg => 'TMR0', freg => 'INTCON', flag => 'T0IF', enable => 'T0IE', ereg => 'INTCON' },
+        TMR1 => { reg => ['TMR1H', 'TMR1L'], freg => 'PIR1', ereg => 'PIE1', flag => 'TMR1IF', enable => 'TMR1E' },
         T0CKI => 17,
         T1CKI => 2,
         T1G => 3,
