@@ -2,9 +2,10 @@ package VIC::PIC::P16F690;
 use strict;
 use warnings;
 
-our $VERSION = '0.23';
+our $VERSION = '0.24';
 $VERSION = eval $VERSION;
 
+use Carp;
 use Moo;
 extends 'VIC::PIC::Base';
 
@@ -256,7 +257,7 @@ has analog_pins => (is => 'ro', default => sub {
             AN8  => [ 8, 8],
             AN9  => [ 9, 9],
             AN10 => [13, 10],
-            AN11 => [12, 12],
+            AN11 => [12, 11],
         }
 });
 
@@ -320,9 +321,10 @@ has wdt_prescaler => (is => 'ro', default => sub {
 
 has timer_pins => (is => 'ro', default => sub {
     {
-        TMR0 => 'TMR0',
-        TMR1 => ['TMR1H', 'TMR1L'],
-        TMR2 => 'TMR2',
+        #reg     #reg      #ireg #flag  #enable
+        TMR0 => { reg => 'TMR0', freg => 'INTCON', flag => 'T0IF', enable => 'T0IE', ereg => 'INTCON' },
+        TMR1 => { reg => ['TMR1H', 'TMR1L'], freg => 'PIR1', ereg => 'PIE1', flag => 'TMR1IF', enable => 'TMR1E' },
+        TMR2 => { reg => 'TMR2', freg => 'PIR1', flag => 'TMR2IF', enable => 'TMR2IE', ereg => 'PIE1' },
         T0CKI => 17,
         T1CKI => 2,
         T1G => 3,
@@ -348,16 +350,27 @@ has eint_pins => (is => 'ro', default => sub {
 
 has ioc_pins => (is => 'ro', default => sub {
     {
-        RA0 => 19,
-        RA1 => 18,
-        RA2 => 17,
-        RA3 => 4,
-        RA4 => 3,
-        RA5 => 2,
-        RB4 => 13,
-        RB5 => 12,
-        RB6 => 11,
-        RB7 => 10,
+               #pin #ioc-bit #ioc-reg
+        RA0 => [19, 'IOCA0', 'IOCA'],
+        RA1 => [18, 'IOCA1', 'IOCA'],
+        RA2 => [17, 'IOCA2', 'IOCA'],
+        RA3 => [4,  'IOCA3', 'IOCA'],
+        RA4 => [3,  'IOCA4', 'IOCA'],
+        RA5 => [2,  'IOCA5', 'IOCA'],
+        RB4 => [13, 'IOCB4', 'IOCB'],
+        RB5 => [12, 'IOCB5', 'IOCB'],
+        RB6 => [11, 'IOCB6', 'IOCB'],
+        RB7 => [10, 'IOCB7', 'IOCB'],
+    }
+});
+
+has ioc_ports => (is => 'ro', default => sub {
+    {
+        # for ports to be used as well
+        PORTA => 'IOCA',
+        PORTB => 'IOCB',
+        FLAG => 'RABIF',
+        ENABLE => 'RABIE',
     }
 });
 
@@ -367,9 +380,71 @@ has usart_pins => (is => 'ro', default => sub {
         async_out => 'TX',
         sync_clock => 'CK',
         sync_data => 'DT',
+        # this defines the port names that the user can use
+        # validly. The port names define whether the user wants to use them in
+        # synchronous or asynchronous mode
+        UART => 'async',
+        USART => 'sync',
     }
 });
 
+sub usart_baudrates {
+    my ($self, $baud, $f_osc, $sync) = @_;
+    $baud = 9600 unless defined $baud;
+    my %acceptable = map { $_ => 1 } (300, 1200, 2400, 9600, 19200, 57600, 115200);
+    unless (exists $acceptable{$baud}) {
+        my $str = sprintf "Baud rate %d is unacceptable. Allowed rates are: %s",
+        $baud, join(',', keys %acceptable);
+        carp $str;
+        return;
+    }
+    $f_osc = defined $f_osc ? $f_osc : $self->f_osc;
+    $sync = defined $sync ? $sync : 0;
+    ## we check with the expected error rates and pick the appropriate baud-rate
+    ## generation parameters such as the values of BRG16 and BRGH
+    if ($sync) {
+        # BRGH = x, BRG16 = 1
+        my $spbrg = int(($f_osc / ($baud * 4)) - 1);
+        my $cbaud = int($f_osc / (($spbrg + 1) * 4));
+        my $error = (($cbaud - $baud) * 100) / $baud;
+        return { SPBRG => $spbrg, BRGH => 0, BRG16 => 1,
+                error => $error, actual => $cbaud, baud => $baud };
+    } else {
+        # BRG16 = 0, BRGH = 0, BRG is 8-bit
+        my $spbrg_00 = int(($f_osc / ($baud * 64)) - 1);
+        my $cbaud_00 = $f_osc / (($spbrg_00 + 1) * 64);
+        my $error_00 = (($cbaud_00 - $baud) * 100) / $baud;
+        my $hh_00 = { SPBRG => $spbrg_00, error => $error_00, actual => $cbaud_00,
+                      BRG16 => 0, BRGH => 0, baud => $baud } if $spbrg_00 < 0x100;
+        # BRG16 = 0, BRGH = 1, BRG is 8-bit
+        my $spbrg_01 = int(($f_osc / ($baud * 16)) - 1);
+        my $cbaud_01 = $f_osc / (($spbrg_01 + 1) * 16);
+        my $error_01 = (($cbaud_01 - $baud) * 100) / $baud;
+        my $hh_01 = { SPBRG => $spbrg_01, error => $error_01, actual => $cbaud_01,
+                      BRG16 => 0, BRGH => 1, baud => $baud } if $spbrg_01 < 0x100;
+        # BRG16 = 1, BRGH = 0, BRG is 16-bit
+        my $spbrg_10 = int(($f_osc / ($baud * 16)) - 1);
+        my $cbaud_10 = $f_osc / (($spbrg_10 + 1) * 16);
+        my $error_10 = (($cbaud_10 - $baud) * 100) / $baud;
+        my $hh_10 = { SPBRG => $spbrg_10, error => $error_10, actual => $cbaud_10,
+                      BRG16 => 1, BRGH => 0, baud => $baud } if $spbrg_10 < 0x10000;
+        # BRG16 = 1, BRGH = 1, BRG is 16-bit
+        my $spbrg_11 = int(($f_osc / ($baud * 4)) - 1);
+        my $cbaud_11 = $f_osc / (($spbrg_11 + 1) * 4);
+        my $error_11 = (($cbaud_11 - $baud) * 100) / $baud;
+        my $hh_11 = { SPBRG => $spbrg_11, error => $error_11, actual => $cbaud_11,
+                      BRG16 => 1, BRGH => 1, baud => $baud } if $spbrg_11 < 0x10000;
+        ## sort based on error in ascending order and remove NaN
+        my @sorted = sort { $a->{error} <=> $b->{error} }
+                     grep { defined $_ and $_->{error} == $_->{error} } ($hh_00, $hh_01, $hh_10, $hh_11);
+        unless (@sorted) {
+            carp "Cannot seem to find appropriate baud generator values for $baud";
+            return;
+        }
+        return wantarray ? @sorted : $sorted[0];
+    }
+}
+
 has selector_pins => (is => 'ro', default => sub {
     {
         'spi_or_i2c' => 'SS',
@@ -427,7 +502,7 @@ has chip_config => (is => 'ro', default => sub {
 });
 
 my @rolenames = qw(CodeGen Operators Chip GPIO ADC ISR Timer Operations ECCP
-                    USART SPI I2C Comparator);
+                    USART SPI I2C Comparator Power);
 my @roles = map (("VIC::PIC::Roles::$_", "VIC::PIC::Functions::$_"), @rolenames);
 with @roles;
 
