@@ -1,7 +1,7 @@
 package VIC::PIC::P18F242;
 use strict;
 use warnings;
-our $VERSION = '0.23';
+our $VERSION = '0.24';
 $VERSION = eval $VERSION;
 use Moo;
 extends 'VIC::PIC::Base';
@@ -388,9 +388,16 @@ has usart_pins => (is => 'ro', default => sub {
         async_out => 'TX',
         sync_clock => 'CK',
         sync_data => 'DT',
+        # this defines the port names that the user can use
+        # validly. The port names define whether the user wants to use them in
+        # synchronous or asynchronous mode
+        UART => 'async',
+        USART => 'sync',
     }
 });
 
+sub usart_baudrates {}
+
 has selector_pins => (is => 'ro', default => sub {
     {
         'spi_or_i2c' => 'SS',
