@@ -1,6 +1,6 @@
 use strict;
 use warnings;
-our $VERSION = '0.23';
+our $VERSION = '0.24';
 $VERSION = eval $VERSION;
 
 package VIC::PIC::Roles::CodeGen;
@@ -8,7 +8,7 @@ package VIC::PIC::Roles::CodeGen;
     use Moo::Role;
     requires qw(type org include chip_config get_chip_config code_config
       validate validate_modifier_operator update_code_config
-      list_roles
+      list_roles is_variable
     );
 }
 
@@ -23,7 +23,7 @@ package VIC::PIC::Roles::Operators;
       op_inc op_dec op_add op_sub op_mul op_div op_mod op_bxor
       op_band op_bor op_eq op_lt op_ge op_ne op_le op_gt op_and
       op_or op_sqrt break continue store_array store_string
-      store_table op_tblidx op_stridx op_arridx
+      store_table op_tblidx op_stridx op_arridx store_bytes
     );
 }
 
@@ -50,7 +50,7 @@ package VIC::PIC::Roles::GPIO;
     # analog_pins are a list of analog_pins
     requires qw(input_pins output_pins io_ports
       analog_pins get_input_pin get_output_pin);
-    requires qw(digital_output digital_input analog_input write);
+    requires qw(digital_output digital_input analog_input write read setup);
 }
 
 package VIC::PIC::Roles::ADC;
@@ -74,7 +74,7 @@ package VIC::PIC::Roles::ISR;
     use Moo::Role;
 
     requires qw(eint_pins ioc_pins);
-    requires qw(isr_entry isr_exit isr_var isr_timer);
+    requires qw(isr_entry isr_exit isr_var isr_timer isr_ioc);
 }
 
 package VIC::PIC::Roles::CCP;
@@ -98,7 +98,7 @@ package VIC::PIC::Roles::Operations;
 package VIC::PIC::Roles::USART;
 {
     use Moo::Role;
-    requires qw(usart_pins);
+    requires qw(usart_pins usart_write usart_read usart_setup usart_baudrates);
 }
 
 package VIC::PIC::Roles::SPI;
@@ -137,5 +137,11 @@ package VIC::PIC::Roles::USB;
     requires qw(usb_pins);
 }
 
+package VIC::PIC::Roles::Power;
+{
+    use Moo::Role;
+    requires qw(sleep);
+}
+
 1;
 __END__
