@@ -6,7 +6,7 @@ use POSIX ();
 use List::Util qw(max);
 use List::MoreUtils qw(any firstidx indexes);
 
-our $VERSION = '0.23';
+our $VERSION = '0.24';
 $VERSION = eval $VERSION;
 
 use Pegex::Base;
@@ -26,6 +26,8 @@ has ast => {
     tmp_variables => {},
     conditionals => 0,
     tmp_stack_size => 0,
+    strings => 0,
+    tables => [],
     asserts => 0,
 };
 has intermediate_inline => undef;
@@ -99,19 +101,21 @@ sub handle_named_block {
     my ($self, $name, $anon_block, $parent) = @_;
     my $id = $1 if $anon_block =~ /_anonblock(\d+)/;
     $id = $self->ast->{block_count} unless defined $id;
-    my $expected_label;
+    my ($expected_label, $expected_param) = ('', '');
     if ($name eq 'Main') {
         $expected_label = "_start";
     } elsif ($name =~ /^Loop/) {
         $expected_label = "_loop_${id}";
     } elsif ($name =~ /^Action/) {
         $expected_label = "_action_${id}";
+        $expected_param = "action${id}_param";
     } elsif ($name =~ /^True/) {
         $expected_label = "_true_${id}";
     } elsif ($name =~ /^False/) {
         $expected_label = "_false_${id}";
     } elsif ($name =~ /^ISR/) {
         $expected_label = "_isr_${id}";
+        $expected_param = "isr${id}_param";
     } elsif ($name eq 'Simulator') {
         $expected_label = '_vic_simulator';
     } else {
@@ -121,10 +125,14 @@ sub handle_named_block {
     $self->ast->{block_mapping}->{$name} = {
         label => $expected_label,
         block => $anon_block,
+        params => [],
+        param_prefix => $expected_param,
     };
     $self->ast->{block_mapping}->{$anon_block} = {
         label => $expected_label,
         block => $name,
+        params => [],
+        param_prefix => $expected_param,
     };
     # make sure the anon-block and named-block refer to the same block
     $self->ast->{$name} = $self->ast->{$anon_block};
@@ -158,6 +166,7 @@ sub handle_named_block {
                 $slabel = "_start_conditional_$ccount";
             }
             $block_label .= "::$elabel";
+            $block_label .= "::$expected_param" if length $expected_param;
             push @{$self->ast->{$parent}}, $block_label;
         }
         # save this for referencing when we need to know what the parent of
@@ -242,10 +251,8 @@ sub got_instruction {
     my @args = ();
     while (scalar @$list) {
         my $a = shift @$list;
-        if ($a =~ /BLOCK::(\w+)::Action\w+::.*::(_end_\w+)$/) {
-            push @args, "ACTION::$1::END::$2";
-        } elsif ($a =~ /BLOCK::(\w+)::ISR\w+::.*::(_end_\w+)$/) {
-            push @args, "ISR::$1::END::$2";
+        if ($a =~ /BLOCK::(\w+)::(Action|ISR)\w+::.*::(_end_\w+)::(\w+)$/) {
+            push @args, uc($2) . "::$1::END::$3::PARAM::$4";
         } else {
             push @args, $a;
         }
@@ -314,6 +321,14 @@ sub got_array_element {
         "Unable to create intermediary variable '$varname'") unless $varname;
 }
 
+sub got_parameter {
+    my $self = shift;
+    ## ok the target variable needs a parameter here
+    ## this works only in block scope so we want to check which block we are in
+    my $block = $self->ast->{block_stack}->[-1];
+    return "PARAM::$block";
+}
+
 sub got_declaration {
     my ($self, $list) = @_;
     my $lhs = shift @$list;
@@ -339,13 +354,20 @@ sub got_declaration {
         } else {
             return $self->parser->throw_error("Variable '$lhs' doesn't exist");
         }
+    } elsif ($rhs =~ /PARAM::(\w+)/) {
+        ## ok now we push this as our statement and handle the rest during
+        ## code generation
+        ## this is of the format PARAM::op::block_name::variable
+        my $block = $1;
+        $self->update_intermediate("PARAM::ASSIGN::${block}::${lhs}");
     } else {
         # var = number | string etc.
         if ($rhs =~ /^-?\d+$/) {
             # we just use the got_assign_expr. this should never be called in
             # reality but is here in case the grammar rules change
-            $self->update_intermetdiate("SET::ASSIGN::${lhs}::${rhs}");
+            $self->update_intermediate("SET::ASSIGN::${lhs}::${rhs}");
         } else {
+            #VIKAS: check this!
             # handle strings here
             $self->ast->{variables}->{$lhs}->{type} = 'string';
             $self->ast->{variables}->{$lhs}->{data} = $rhs;
@@ -718,6 +740,40 @@ sub got_boolean {
     return 0; # default boolean is false
 }
 
+sub got_double_quoted_string {
+    my $self = shift;
+    my $str = pop;
+    ## Ripped from Ingy's pegex-json-pm Pegex::JSON::Data
+    ## Unicode support not implemented yet but available in Pegex::JSON::Data
+    my %escapes = (
+        '"' => '"',
+        '/' => '/',
+        "\\" => "\\",
+        b => "\b",
+        f => "\x12",
+        n => "\n",
+        r => "\r",
+        t => "\t",
+        0 => "\0",
+    );
+    $str =~ s/\\(["\/\\bfnrt0])/$escapes{$1}/ge;
+    return $str;
+}
+
+sub got_string {
+    my $self = shift;
+    my $str = shift;
+    my $stref = {
+        string => $str,
+        block => $self->ast->{block_stack}->[-1],
+        name => sprintf("_vic_str_%02d", $self->ast->{strings}),
+    };
+    $self->global_collections->{"$stref"} = $stref;
+    $self->ast->{strings}++;
+    return $stref;
+    #return '@' . $str;
+}
+
 sub got_number {
     my ($self, $list) = @_;
     # if it is a hexadecimal number we can just convert it to number using int()
@@ -775,6 +831,23 @@ sub _update_funcs {
     1;
 }
 
+sub _update_tables {
+    my ($self, $tables) = @_;
+    if (ref $tables eq 'HASH') {
+        $tables = [ $tables ];
+    }
+    unless (ref $tables eq 'ARRAY') {
+        return $self->parser->throw_error(
+        "Code generation error. PIC methods should return strings as a HASH or ARRAY");
+    }
+    foreach my $s (@$tables) {
+        next unless defined $s->{bytes};
+        next unless defined $s->{name};
+        push @{$self->ast->{tables}}, $s;
+    }
+    1;
+}
+
 ## assert handling is special for now
 sub got_assert_comparison {
     my ($self, $list) = @_;
@@ -826,11 +899,12 @@ sub generate_code_instruction {
         next unless exists $self->global_collections->{$_};
         $_ = $self->global_collections->{$_};
     }
-    my ($code, $funcs, $macros) = $self->pic->$method(@ins);
+    my ($code, $funcs, $macros, $tables) = $self->pic->$method(@ins);
     return $self->parser->throw_error("Error in intermediate code '$line'") unless $code;
     push @code, "\t;; $line" if $self->intermediate_inline;
     push @code, $code if $code;
     $self->_update_funcs($funcs, $macros) if ($funcs or $macros);
+    $self->_update_tables($tables) if $tables;
     return @code;
 }
 
@@ -844,11 +918,12 @@ sub generate_code_unary_expr {
     # check if temporary variable or not
     if (exists $ast->{variables}->{$varname}) {
         my $nvar = $ast->{variables}->{$varname}->{name} || $varname;
-        my ($code, $funcs, $macros) = $self->pic->$method($nvar);
+        my ($code, $funcs, $macros, $tables) = $self->pic->$method($nvar);
         return $self->parser->throw_error("Error in intermediate code '$line'") unless $code;
         push @code, "\t;; $line" if $self->intermediate_inline;
         push @code, $code if $code;
         $self->_update_funcs($funcs, $macros) if ($funcs or $macros);
+        $self->_update_tables($tables) if $tables;
     } else {
         return $self->parser->throw_error("Error in intermediate code '$line'");
     }
@@ -889,10 +964,11 @@ sub generate_code_operations {
     $self->parser->throw_error("Invalid operator '$op' in intermediate code") unless
         ($method and $self->pic->can($method));
     push @code, "\t;; $line" if $self->intermediate_inline;
-    my ($code, $funcs, $macros) = $self->pic->$method($var1, $var2, %extra);
+    my ($code, $funcs, $macros, $tables) = $self->pic->$method($var1, $var2, %extra);
     return $self->parser->throw_error("Error in intermediate code '$line'") unless $code;
     push @code, $code if $code;
     $self->_update_funcs($funcs, $macros) if ($funcs or $macros);
+    $self->_update_tables($tables) if $tables;
     return @code;
 }
 
@@ -967,7 +1043,7 @@ sub generate_code_assign_expr {
     my @code = ();
     my $ast = $self->ast;
     my ($tag, $op, $varname, $rhs) = split /::/, $line;
-    push @code, ";;; $line\n";
+    push @code, ";;; $line\n" if $self->intermediate_inline;
     if (exists $ast->{variables}->{$varname}) {
         if (exists $ast->{tmp_variables}->{$rhs}) {
             my $tmp_code = $ast->{tmp_variables}->{$rhs};
@@ -1015,11 +1091,12 @@ sub generate_code_assign_expr {
             my $method = $self->pic->validate_operator($op);
             $self->parser->throw_error("Invalid operator '$op' in intermediate code") unless $self->pic->can($method);
             my $nvar = $ast->{variables}->{$varname}->{name} || $varname;
-            my ($code, $funcs, $macros) = $self->pic->$method($nvar, $rhs);
+            my ($code, $funcs, $macros, $tables) = $self->pic->$method($nvar, $rhs);
             return $self->parser->throw_error("Error in intermediate code '$line'") unless $code;
             push @code, "\t;; $line" if $self->intermediate_inline;
             push @code, $code if $code;
             $self->_update_funcs($funcs, $macros) if ($funcs or $macros);
+            $self->_update_tables($tables) if $tables;
         }
     } else {
         return $self->parser->throw_error(
@@ -1236,13 +1313,36 @@ sub generate_code {
         my $line = shift @$blocks;
         next unless defined $line;
         if ($line =~ /^BLOCK::\w+/) {
-            push @code, $self->generate_code_blocks($line, $block_name);
+            my $blockparams = $ast->{block_mapping}->{$block_name}->{params} || [];
+            push @code, $self->generate_code_blocks($line, $block_name, $blockparams);
         } elsif ($line =~ /^INS::\w+/) {
             push @code, $self->generate_code_instruction($line);
         } elsif ($line =~ /^UNARY::\w+/) {
             push @code, $self->generate_code_unary_expr($line);
         } elsif ($line =~ /^SET::\w+/) {
             push @code, $self->generate_code_assign_expr($line);
+        } elsif ($line =~ /^PARAM::(\w+)::(\w+)::(\w+)/) {
+            if (exists $ast->{block_mapping}->{$block_name}) {
+                my $op = $1;
+                my $pblock = $2;
+                my $pvar = $3;
+                my $mapping = $ast->{block_mapping}->{$pblock};
+                my $param_idx = scalar @{$mapping->{params}};
+                my $paramvar = $mapping->{param_prefix} || lc($block_name . '_param');
+                $paramvar .= $param_idx;
+                push @{$mapping->{params}}, $paramvar;
+                # map the param index back to the other mapping too
+                if ($pblock ne $block_name and $mapping->{block} eq $block_name) {
+                    my $mapping2 = $ast->{block_mapping}->{$block_name};
+                    $mapping2->{params} = $mapping->{params};
+                }
+                my $pline = "SET::${op}::${pvar}::${paramvar}";
+                #YYY [$pblock, $pvar, $block_name, $param_idx, $pline, $paramvar];
+                push @code, $self->generate_code_assign_expr($pline);
+            } else {
+                $self->parser->throw_error("Intermediate code '$line' in block "
+                    . "$block_name cannot be handled");
+            }
         } elsif ($line =~ /^LABEL::(\w+)/) {
             my $lbl = $1;
             push @code, ";; $line" if $self->intermediate_inline;
@@ -1361,7 +1461,12 @@ sub final {
             $funcs .= "\n";
         }
     }
+    foreach my $tbl (@{$ast->{tables}}) {
+        my $dt = $tbl->{bytes};
+        my $dn = $tbl->{name};
+    }
     $funcs .= join ("\n", @tables) if scalar @tables;
+    $funcs .= $self->pic->store_bytes($ast->{tables});
     if (length $isr_code) {
         my $isr_entry = $self->pic->isr_entry;
         my $isr_exit = $self->pic->isr_exit;
